#ifndef _CONFIG_H_
#define _CONFIG_H_

/*
 * Copyright (C) 2006-2007 by egnite Software GmbH
 * Copyright (C) 2008-2009 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file config.h
 * \brief Configuration items.
 *
 * \verbatim
 * $Id: config.h 65 2010-08-01 15:33:24Z haraldkipp $
 * \endverbatim
 */

/*
 * Include Nut/OS configuration files.
 */
#include <cfg/os.h> /* Nut/OS kernel settings. */
#include <cfg/clock.h> /* Hardware clock settings. */
#include <cfg/memory.h> /* Memory layout. */
#include <cfg/lcd.h> /* LCD settings. */

#include <dev/board.h>
#include <stdint.h>
#include <stdio.h>

#ifndef VERSION
#define VERSION "2.2.1"
#endif

/*
 * What is available.
 * ========================================================
 */

/*
 * Include hardware dependent radio specifications.
 */
#if defined(ETHERNUT2)
#include "ethernut2.h"
#elif defined(ETHERNUT3)
#include "ethernut3.h"
#elif defined(ELEKTOR_IR1)
#include "eir1.h"
#elif defined(AT91SAM7X_EK)
#include "sam7xek.h"
#elif defined(AT91SAM9260_EK)
#include "sam9260ek.h"
#elif defined(ALTERA_DE0_NANO)
#include "de0_nano.h"
#elif defined(ALTERA_DE1)
#include "de1.h"
#endif
/* Always include generic configuration to fix missing items. */
#include "hardware.h"

/*
 * What should be used.
 * ========================================================
 */
/*! \brief Wether we should run a discovery responder. */
#ifndef USE_DISCOVERY
#define USE_DISCOVERY       0
#endif

/*! \brief Wether we should query a time daemon. */
#ifndef USE_TIMED
#define USE_TIMED           0
#endif

/*! \brief Wether we should use the character display. */
#ifndef USE_CHAR_DISPLAY
/*!
 * \brief Enable or disable character display control.
 *
 * To route the LCD output to stdout, use
 * \code
 * #define USE_TERM  1
 * \endcode
 * Seems to work with Hyperterminal, but not with TeraTerm.
 * Note, that you must also set
 * \code
 * #define LOG_ENABLE 0
 * \endcode
 * in logmsg.h.
 */
#define USE_CHAR_DISPLAY    RADIO_HAS_CHAR_DISPLAY
#endif

/*! \brief Wether we should use the ATmega driven display. */
#ifndef USE_MEGA_DISPLAY
#define USE_MEGA_DISPLAY    RADIO_HAS_MEGA_DISPLAY
#endif

/*! \brief Wether we should monitor push buttons. */
#ifndef USE_PUSHBUTTONS
#define USE_PUSHBUTTONS     RADIO_HAS_PUSHBUTTONS
#endif

/*! \brief Wether to use the hardware codec. */
#ifndef USE_HARDWARE_CODEC
#define USE_HARDWARE_CODEC  RADIO_HAS_HARDWARE_CODEC
#endif

/*! \brief Wether to use the HTTP interface. */
#ifndef USE_HTTPSERVER
#define USE_HTTPSERVER      1
#endif

/*! \brief Wether to use dynamically created threads. */
#ifndef USE_DYNAMIC_THREADS
#define USE_DYNAMIC_THREADS 0
#endif

#ifndef USE_LIST_SORTING
#define USE_LIST_SORTING    RADIO_HAS_LARGE_RAM
#endif

/*! 
 * \brief Level of debug output.
 *
 * Some useful output can be directed to the debug device, typically the
 * RS-232 interface.
 *
 * Set to zero to save memory. Otherwise combine any of the following 
 * flags:
 *
 * - LOG_ERROR to print error messages.
 * - LOG_WARN to print warnings.
 * - LOG_STARTUP to print messages during startup.
 * - LOG_CONFIG to print messages when the configuration changes.
 * - LOG_STATION
 * - LOG_SHOUTCAST
 * - LOG_USERIF
 * - LOG_HTTPD
 * - LOG_XMLD
 * - LOG_XMLD
 *
 * Furthermore, LOG_DEBUG may be added to increase verbosity. But this
 * will consume even more data and program memory.
 */
#ifndef LOG_ENABLE
#define LOG_ENABLE      (LOG_STARTUP | LOG_ERROR | LOG_WARN)
#endif

/*
 * General system settings.
 * ========================================================
 */

/* \brief Local timezone, -1 for Central Europe. */
#ifndef RADIO_DEFAULT_TIMEZONE
#define RADIO_DEFAULT_TIMEZONE    -1
#endif


/*
 * Multithreading configuration.
 * ========================================================
 */

/*!\brief Default multiplier. */
#ifndef NUT_THREAD_STACK_MULT
#define NUT_THREAD_STACK_MULT   1
#endif

/*!\brief Default offset. */
#ifndef NUT_THREAD_STACK_ADD
#define NUT_THREAD_STACK_ADD    0
#endif

#ifndef SHOUTCAST_THREAD_STACK
#if defined(__AVR__)
/* NUTDEBUG_CHECK_STACK reported 92 bytes available with optimized code. */
#define SHOUTCAST_THREAD_STACK  ((256 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#elif defined(__arm__)
#if USE_HARDWARE_CODEC
/* 272 bytes optimized, 644 bytes debug code. */
#define SHOUTCAST_THREAD_STACK  ((384 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#else
/* Had been 716 bytes optimized code. But later test on SAM7X was 272 bytes. */
#define SHOUTCAST_THREAD_STACK  ((384 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#endif
#else
#if USE_HARDWARE_CODEC
#define SHOUTCAST_THREAD_STACK  ((NUT_THREAD_MAINSTACK * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#else
#define SHOUTCAST_THREAD_STACK  ((2 * NUT_THREAD_MAINSTACK * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#endif
#endif
#endif

#ifndef UI_THREAD_STACK
#if defined(__AVR__)
/* Untested. */
#define UI_THREAD_STACK ((384 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#else
/* 236 bytes optimized code. */
#define UI_THREAD_STACK ((384 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#endif
#endif

/* Server thread stack size. */
#ifndef HTTPD_SERVICE_STACK
#if defined(__AVR__)
/* 476 bytes optimized. */
#define HTTPD_SERVICE_STACK ((580 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#elif defined(__arm__)
/* 892 bytes optimized, 1520 bytes debug code. */
#define HTTPD_SERVICE_STACK ((1024 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#else
#define HTTPD_SERVICE_STACK  ((2 * NUT_THREAD_MAINSTACK * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#endif
#endif

/* Number of server threads always running. */
#ifndef HTTPD_MIN_THREADS
#if RADIO_HAS_LARGE_RAM
#define HTTPD_MIN_THREADS   8
#else
#define HTTPD_MIN_THREADS   4
#endif
#endif

/* Maximum number of server threads. */
#ifndef HTTPD_MAX_THREADS
#define HTTPD_MAX_THREADS   16
#endif

#ifndef XMLD_SERVICE_STACK
#if defined(__AVR__)
/* NUTDEBUG_CHECK_STACK reported 88 bytes available. */
#define XMLD_SERVICE_STACK ((256 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#elif defined(__arm__)
/* 352 bytes optimized, 712 bytes debug code. */
#define XMLD_SERVICE_STACK ((448 * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#else
#define XMLD_SERVICE_STACK  ((NUT_THREAD_MAINSTACK * NUT_THREAD_STACK_MULT) + NUT_THREAD_STACK_ADD)
#endif
#endif

/* Number of report server threads always running. */
#ifndef XMLD_MIN_THREADS
#ifdef __AVR__
#define XMLD_MIN_THREADS   2
#else
#define XMLD_MIN_THREADS   4
#endif
#endif

/* Maximum number of report server threads. */
#ifndef XMLD_MAX_THREADS
#define XMLD_MAX_THREADS   8
#endif


/*
 * Basic network configuration.
 * ========================================================
 */

/*
 * Unique MAC address of the Ethernut Board.
 *
 * Ignored if EEPROM contains a valid configuration.
 */
#define RADIO_DEFAULT_MAC { 0x00, 0x06, 0x98, 0x4F, 0x00, 0x00 }

/*
 * Unique IP address of the Ethernut Board.
 *
 * Ignored if DHCP is used.
 */
#define RADIO_DEFAULT_IPADDR "192.168.192.39"

/*
 * IP network mask of the Ethernut Board.
 *
 * Ignored if DHCP is used.
 */
#define RADIO_DEFAULT_IPMASK "255.255.255.0"

/*
 * Gateway IP address for the Ethernut Board.
 *
 * Ignored if DHCP is used.
 */
#define RADIO_DEFAULT_IPGATE "192.168.192.1"

/*
 * IP address of the host running a time daemon. 
 */
#define RADIO_DEFAULT_TIMED "130.149.17.21"

/*
 * Maximum segment size, choose 536 up to 1460. Note, that segment 
 * sizes above 536 may result in fragmented packets. Remember, that 
 * Nut/Net doesn't support TCP fragmentation.
 */
#ifndef MAX_TCPSEG_SIZE
#if RADIO_HAS_LARGE_RAM
#define MAX_TCPSEG_SIZE 1460
#else
#define MAX_TCPSEG_SIZE 536
#endif
#endif

/*
 * TCP buffer size.
 */
#ifndef MAX_TCPBUF_SIZE
#if RADIO_HAS_LARGE_RAM
#define MAX_TCPBUF_SIZE (12 * MAX_TCPSEG_SIZE)
#else
#define MAX_TCPBUF_SIZE (3 * MAX_TCPSEG_SIZE)
#endif
#endif

/*
 * Socket receive timeout.
 */
#ifndef MAX_TCPRCV_WAIT
#define MAX_TCPRCV_WAIT     5000
#endif

#ifndef MAX_HEADERLINE_SIZE
#define MAX_HEADERLINE_SIZE     255
#endif

/*
 * Software decoder configuration.
 * ========================================================
 */

#ifndef MP3_BUFSIZ
#if RADIO_HAS_LARGE_RAM
#define MP3_BUFSIZ          1048576
#else
#define MP3_BUFSIZ          (4 * MAINBUF_SIZE)
#endif
#endif

#ifndef MAX_WAITSTREAM
#define MAX_WAITSTREAM      20
#endif

#ifndef MAX_WAIT_MP3BUF_FILLED
#define MAX_WAIT_MP3BUF_FILLED  2
#endif


/*
 * SHOUTcast configuration.
 * ========================================================
 */

#ifndef SHOUTCAST_READ_BUFSIZ
#define SHOUTCAST_READ_BUFSIZ   512
#endif

//#define SHOUTCAST_QLIST "www.shoutcast.com/sbin/newxml.phtml"
#define SHOUTCAST_QLIST "yp.shoutcast.com/sbin/newxml.phtml"

//#define SHOUTCAST_QTUNE "www.shoutcast.com/sbin/tunein-station.pls"
#define SHOUTCAST_QTUNE "yp.shoutcast.com/sbin/tunein-station.pls"

/* Currently we have Icecast/SHOUTcast only. */
#ifndef MAX_RECEIVERS
#define MAX_RECEIVERS   1
#endif

#ifndef STATION_MAX_SEGSIZE
#define STATION_MAX_SEGSIZE 1460
#endif

#ifndef STATION_TCP_BUFSIZE
#define STATION_TCP_BUFSIZE (6 * STATION_MAX_SEGSIZE)
#endif

#ifndef STATION_TCP_TIMEOUT
#define STATION_TCP_TIMEOUT 2000
#endif

#ifndef STATION_QUERY_LIMIT
#if RADIO_HAS_LARGE_RAM == 0
#define STATION_QUERY_LIMIT 10
#else
#define STATION_QUERY_LIMIT 0
#endif
#endif


/*
 * HTTP daemon configuration.
 * ========================================================
 */

#ifndef RADIO_FSDEV_HTTP
#define RADIO_FSDEV_HTTP    devUrom
#endif

#ifdef RADIO_FSNAME_HTTP
#define RADIO_HTTP_ROOT     RADIO_FSNAME_HTTP ":/"
#endif

/* The TCP port we are listening at. */
#ifndef HTTPD_TCP_PORT
#define HTTPD_TCP_PORT      80
#endif

#ifndef HTTPD_MAX_SEGSIZE
#define HTTPD_MAX_SEGSIZE   MAX_TCPSEG_SIZE
#endif

/* TCP stream buffer size. */
#ifndef HTTPD_TCP_BUFSIZE
#define HTTPD_TCP_BUFSIZE   MAX_TCPBUF_SIZE
#endif

/* Closes keep-alive connections. */
#ifndef HTTPD_TCP_TIMEOUT
#define HTTPD_TCP_TIMEOUT   500
#endif


/*
 * XML daemon configuration.
 * ========================================================
 */

/* The TCP port we are sending status changes. */
#ifndef XMLD_REPO_TCP_PORT
#define XMLD_REPO_TCP_PORT      2008
#endif

/* The TCP port we are receiving control commands. */
#ifndef XMLD_CMD_TCP_PORT
#define XMLD_CMD_TCP_PORT       2009
#endif

/*
 * Maximum segment size.
 */
#ifndef XMLD_MAX_SEGSIZE
#define XMLD_MAX_SEGSIZE   MAX_TCPSEG_SIZE
#endif

/* TCP stream buffer size. */
#ifndef XMLD_TCP_BUFSIZE
#define XMLD_TCP_BUFSIZE   MAX_TCPBUF_SIZE
#endif

/* Detects broken connections. */
#ifndef XMLD_TCP_TIMEOUT
#define XMLD_TCP_TIMEOUT   MAX_TCPRCV_WAIT
#endif


/*
 * Non-volatile memory configuration.
 * ========================================================
 */

/*! 
 * \brief Whether we should use the raw file system for configuration data.
 *
 * Right now there are no alternatives other than storing the
 * configuration data in a serial Dataflash chip, using the
 * raw file system driver.
 *
 * Later we may support PHAT on MMC.
 */
#ifndef USE_CONFIG_RAWFS
#define USE_CONFIG_RAWFS    RADIO_HAS_SERIAL_FLASH
#endif

#define CONFIG_MAGIC    "ir1"

#ifndef NVMEM_CONFIG_POS
#define NVMEM_CONFIG_POS    0
#endif


#define FAVLIST_MAGIC   "FL1"

#ifndef NVMEM_FAVLIST_POS
#define NVMEM_FAVLIST_POS   528
#endif

/*! \brief Maximum number of favorites. */
#ifndef MAX_FAVORITES
#if RADIO_HAS_LARGE_RAM
#define MAX_FAVORITES       128
#else
#define MAX_FAVORITES       8
#endif
#endif

/*! \brief Maximum number of URI entries per favorite. */
#ifndef MAXNUM_STREAMS
#if RADIO_HAS_LARGE_RAM
#define MAXNUM_STREAMS      32
#else
#define MAXNUM_STREAMS      4
#endif
#endif

/*
 * User interface configuration.
 * ========================================================
 */

#ifndef LCD_ROWS
#define LCD_ROWS    2
#endif

#ifndef LCD_COLS
#define LCD_COLS    16
#endif

/*!
 * \def LCD_VCOLS
 * \brief Number of virtual columns.
 *
 * Contents will be scrolled, if it doesn't fit on a single LCD line.
 */
#ifndef LCD_VCOLS
#ifdef RADIO_HAS_LARGE_RAM
#define LCD_VCOLS    160
#else
#define LCD_VCOLS    40
#endif
#endif

#if USE_CHAR_DISPLAY || USE_TERM
#ifndef LCD_SCROLL_GAP
#define LCD_SCROLL_GAP  (LCD_COLS / 2)
#endif
#endif

#ifndef LCD_SHORT_DELAY
#define LCD_SHORT_DELAY 100
#endif

#ifndef LCD_LONG_DELAY
#define LCD_LONG_DELAY  1000
#endif


#ifndef UI_REFRESH_RATE
#define UI_REFRESH_RATE 300
#endif



__BEGIN_DECLS
/* Prototypes */

extern FILE *ConfigOpen(void);
extern int ConfigLoad(FILE *fp);
extern int ConfigSave(FILE *fp);

extern size_t ConfigLoadString(FILE *fp, char * str, size_t size);
extern void ConfigResetFactory(void);
extern size_t ConfigSize(void);

__END_DECLS
/* End of prototypes */
#endif
