/*!
 * Copyright (C) 2006-2007 by egnite Software GmbH
 * Copyright (C) 2008-2009 by egnite GmbH
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file config.c
 * \brief Radio configuration.
 *
 * \verbatim
 * $Id: favlist.c 64 2010-08-01 15:32:22Z haraldkipp $
 * \endverbatim
 */


#include "config.h" /* Must be first. */

#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#include <sys/version.h>
#include <sys/confnet.h>
#include <sys/atom.h>
#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/socket.h>

#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <pro/dhcp.h>

#include <memdebug.h>

#include "logmsg.h"
#include "utils.h"
#include "favlist.h"
#include "webradio.h"

RADIOSTATION favlist[MAX_FAVORITES];

/*!
 * \brief Clear favorites list.
 *
 * Will be called by FavListLoad() before loading a new list.
 */
static void FavListClear(void)
{
    int idx;
    int i;

    LogDbgMsg(LOG_CONFIG, "Release list\n");
    /* Release all memory occupied by the current configuration. */
    for (idx = 0; idx < MAX_FAVORITES; idx++) {
        if (favlist[idx].rs_name) {
            free(favlist[idx].rs_name);
        }
        for (i = 0; i < favlist[idx].rs_streams; i++) {
            if (favlist[idx].rs_uri[i]) {
                free(favlist[idx].rs_uri[i]);
            }
        }
    }
    memset(favlist, 0, MAX_FAVORITES * sizeof(RADIOSTATION));
}

/*!
 * \brief Set favorite list entry.
 *
 * \param idx  Index of the entry. Use MAX_FAVORITES to add a new entry.
 * \param name Name of the station. Must be provided, if a new station
 *             is added. May be NULL when adding a URI. When used with
 *             an existing entry, the current name will be replaced.
 * \param uri  URI of the stream, that will be added. If NULL, then all
 *             existing entries are cleared. If both, the name and the
 *             URI are NULL, then the list entry is cleared.
 *
 * \return Entry index on success, -1 otherwise.
 */
int FavListSet(int idx, CONST char *name, CONST char * uri)
{
    int i;

    if (idx >= MAX_FAVORITES) {
        /* New entry. Find a free slot. */
        idx = LAST_FAVORITE + 1;
        while (idx < MAX_FAVORITES && favlist[idx].rs_name) {
            idx++;
        }
    }
    if (idx >= MAX_FAVORITES) {
        /* We are full. */
        LogMsg(LOG_ERROR | LOG_CONFIG, "Favorites list full");
        return -1;
    }

    LogMsg(LOG_CONFIG, "Set %d,%s,%s\n", idx, name ? name : "", uri ? uri : "");
    /* 
     * Replace or delete the name. 
     */
    if (name || uri == NULL) {
        if (favlist[idx].rs_name) {
            free(favlist[idx].rs_name);
            favlist[idx].rs_name = NULL;
        }
        if (name) {
            favlist[idx].rs_name = strdup(name);
        }
    }

    /*
     * Add a URI.
     */
    if (uri) {
        if (favlist[idx].rs_streams < MAXNUM_STREAMS) {
            favlist[idx].rs_uri[favlist[idx].rs_streams] = strdup(uri);
            favlist[idx].rs_streams++;
        }
        else {
            LogMsg(LOG_ERROR | LOG_CONFIG, "URI list full");
            idx = -1;
        }
    }

    /*
     * Delete URIs.
     */
    else {
        for (i = 0; i < favlist[idx].rs_streams; i++) {
            if (favlist[idx].rs_uri[i]) {
                free(favlist[idx].rs_uri[i]);
                favlist[idx].rs_uri[i] = NULL;
            }
        }
        favlist[idx].rs_streams = 0;
    }
    return idx;
}

/*!
 * \brief Copy a favorites list entry.
 *
 * \param src Index of the source entry.
 * \param dst Index of the destination entry.
 *
 * \return 0 on success, -1 otherwise.
 */
int FavListCopy(int src, int dst)
{
    int i;

    /*
     * Release an existing entry at the destination index.
     */
    LogMsg(LOG_CONFIG, "Copying %d to %d\n", src, dst);
    if (favlist[dst].rs_name) {
        free(favlist[dst].rs_name);
    }
    for (i = 0; i < favlist[dst].rs_streams; i++) {
        if (favlist[dst].rs_uri[i]) {
            free(favlist[dst].rs_uri[i]);
        }
    }
    memset(&favlist[dst], 0, sizeof(RADIOSTATION));

    /*
     * Copy the source entry to the destination index.
     */
    if (favlist[src].rs_name) {
        favlist[dst].rs_name = strdup(favlist[src].rs_name);
        for (i = 0; i < favlist[src].rs_streams; i++) {
            if (favlist[src].rs_uri[i]) {
                favlist[dst].rs_uri[favlist[dst].rs_streams++] = strdup(favlist[src].rs_uri[i]);
            }
        }
        return 0;
    }
    return -1;
}

/*!
 * \brief Search favorite radio station.
 *
 * \param idx Start index.
 * \param dir Direction to search, 1 (forward) or -1 (backwards).
 *
 * \return Index of the new station or -1 if none available.
 */
int FavListSearch(int idx, int dir)
{
    int rc = idx;

    LogMsg(LOG_CONFIG, "Searching %d %d\n", idx, dir);
    for (;;) {
        rc += dir;
        if (rc <= LAST_FAVORITE) {
            rc = MAX_FAVORITES - 1;
        }
        else if (rc >= MAX_FAVORITES) {
            rc = LAST_FAVORITE + 1;
        }
        if ((favlist[rc].rs_name && favlist[rc].rs_streams)) {
            /* New entry found. */
            break;
        }
        if (rc == idx) {
            /* Whole list searched, but no entry found. */
            return -1;
        }
    }
    return rc;
}

#if USE_CONFIG_RAWFS

/*!
 * \brief Calculate total size of the favorites list.
 *
 * \return Number of bytes used.
 */
size_t FavListSize(void)
{
    size_t rc = 0;
    int idx;
    int i;
    RADIOSTATION *flp;

    for (idx = 1; idx < MAX_FAVORITES; idx++) {
        flp = &favlist[idx];
        if (flp->rs_name) {
            rc += strlen(flp->rs_name) + 1;
            for (i = 0; i < flp->rs_streams; i++) {
                if (flp->rs_uri[i]) {
                    rc += strlen(flp->rs_uri[i]) + 1;
                }
            }
        }
    }
    return rc + 1;
}

/*!
 * \brief Save favorites list in non-volatile memory.
 *
 * \return 0 on success, -1 otherwise.
 */
int FavListSave(FILE *fp)
{
    int idx;
    int i;
    RADIOSTATION *rsp;

    LogDbgMsg(LOG_CONFIG, "Save list --------------\n");
    /* Rewind and save the magic name. */
    fseek(fp, NVMEM_FAVLIST_POS, SEEK_SET);
    fwrite(FAVLIST_MAGIC, 1, sizeof(FAVLIST_MAGIC), fp);

    /* Save stations. */
    for (idx = LAST_FAVORITE; idx < MAX_FAVORITES; idx++) {
        rsp = &favlist[idx];
        if (rsp->rs_name) {
            LogMsg(LOG_CONFIG, "Saving %d %s\n", idx, rsp->rs_name);
            fwrite(rsp->rs_name, 1, strlen(rsp->rs_name) + 1, fp);
            for (i = 0; i < rsp->rs_streams; i++) {
                if (rsp->rs_uri[i]) {
                    LogMsg(LOG_CONFIG, "  URI=%s\n", rsp->rs_uri[i]);
                    fwrite(rsp->rs_uri[i], 1, strlen(rsp->rs_uri[i]) + 1, fp);
                }
            }
            fputc(0, fp);
        }
        else if (idx == LAST_FAVORITE) {
            fwrite("None\x00", 1, 6, fp);
        }
        NutThreadYield();
    }
    fputc(0, fp);

    return fflush(fp);
}

/*!
 * \brief Load favorites list from non-volatile memory.
 *
 * \return 0 on success, -1 otherwise.
 */
int FavListLoad(FILE *fp)
{
    int rc = -1;
    char *buf;
    int idx;
    int i;

    LogDbgMsg(LOG_CONFIG, "Load list --------------\n");
    buf = malloc(255);
    fseek(fp, NVMEM_FAVLIST_POS, SEEK_SET);
    ConfigLoadString(fp, buf, 4);
    LogDbgMsg(LOG_CONFIG, "Magic = %02x %02x %02x %02x\n", buf[0], buf[1], buf[2], buf[3]);
    if (strcmp(buf, FAVLIST_MAGIC) == 0) {
        FavListClear();
        rc = 0;

        for (idx = LAST_FAVORITE; idx < MAX_FAVORITES; idx++) {
            ConfigLoadString(fp, buf, 255);
            if (*buf == 0) {
                break;
            }
            favlist[idx].rs_name = strdup(buf);
            LogMsg(LOG_CONFIG, "Loading %d %s\n", idx, favlist[idx].rs_name);
            for (i = 0; i <= 128; i++) {
                ConfigLoadString(fp, buf, 255);
                if (*buf == 0) {
                    break;
                }
                LogMsg(LOG_CONFIG, "  URI=%s\n", buf);
                if (i < MAXNUM_STREAMS) {
                    favlist[idx].rs_uri[i] = strdup(buf);
                    favlist[idx].rs_streams++;
                }
            }
        }
    }
    free(buf);

    return rc;
}
#else
int FavListSave(FILE *fp)
{
    return -1;
}
#endif

/*!
 * \brief Set pre-configured favorites list.
 */
void FavListResetFactory(void)
{
    int idx;

    LogDbgMsg(LOG_CONFIG, "Reset list --------------\n");
    FavListClear();

    /* 
     * Add pre-configured radio stations. 
     */
#if RADIO_HAS_LARGE_RAM
    if ((idx = FavListSet(MAX_FAVORITES, "FFH", "mp3.ffh.de/radioffh/hqlivestream.mp3")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "mp3.ffh.de/radioffh/livestream.mp3");


    if ((idx = FavListSet(MAX_FAVORITES, "FREQUENCE3", "nono.frequence3.net:8000")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "insomnie.frequence3.net:8000");
    FavListSet(idx, NULL, "frequence3.GW.opentransit.net:8000");
    FavListSet(idx, NULL, "stream-hautdebit.frequence3.net:8000");
    FavListSet(idx, NULL, "pluton.relay.frequence3.net:9000");
    FavListSet(idx, NULL, "193.251.154.242:8000");
    FavListSet(idx, NULL, "88.191.135.182:8000");
    FavListSet(idx, NULL, "193.251.154.243:8000");
    FavListSet(idx, NULL, "193.251.154.245:8000");
    FavListSet(idx, NULL, "193.251.154.244:8000");
    FavListSet(idx, NULL, "193.222.128.174:8000");
    /* Start with this reliable station. */
    FavListCopy(idx, LAST_FAVORITE);

    if ((idx = FavListSet(MAX_FAVORITES, "FREQUENCE3 Ogg", "ogg.frequence3.net:19000/frequence3.ogg")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "ganymede.frequence3.net:19000/frequence3.ogg");
    FavListSet(idx, NULL, "aspairt.frequence3.net:19000/frequence3.ogg");
    FavListSet(idx, NULL, "mao.frequence3.net:19000/frequence3.ogg");
    FavListSet(idx, NULL, "pluton.frequence3.net:19000/frequence3.ogg");
    FavListSet(idx, NULL, "dingo.frequence3.net:8000/frequence3.ogg");

    if ((idx = FavListSet(MAX_FAVORITES, "Radio Swiss Pop", "glb-stream11.streaming.init7.net:80/1/rsp/mp3_128")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "glb-stream12.streaming.init7.net:80/1/rsp/mp3_128");
    FavListSet(idx, NULL, "zlz-stream10.streaming.init7.net:80/1/rsp/mp3_128");
    FavListSet(idx, NULL, "zlz-stream13.streaming.init7.net:80/1/rsp/mp3_128");
    FavListSet(idx, NULL, "zlz-stream12.streaming.init7.net:80/1/rsp/mp3_128");
    FavListSet(idx, NULL, "zlz-stream11.streaming.init7.net:80/1/rsp/mp3_128");
    FavListSet(idx, NULL, "glb-stream13.streaming.init7.net:80/1/rsp/mp3_128");
    FavListSet(idx, NULL, "glb-stream10.streaming.init7.net:80/1/rsp/mp3_128");


    if ((idx = FavListSet(MAX_FAVORITES, "Radio Swiss Classic", "glb-stream10.streaming.init7.net:80/1/rsc_de/mp3_128")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "glb-stream13.streaming.init7.net:80/1/rsc_de/mp3_128");
    FavListSet(idx, NULL, "zlz-stream10.streaming.init7.net:80/1/rsc_de/mp3_128");
    FavListSet(idx, NULL, "zlz-stream12.streaming.init7.net:80/1/rsc_de/mp3_128");
    FavListSet(idx, NULL, "zlz-stream11.streaming.init7.net:80/1/rsc_de/mp3_128");
    FavListSet(idx, NULL, "glb-stream12.streaming.init7.net:80/1/rsc_de/mp3_128");
    FavListSet(idx, NULL, "zlz-stream13.streaming.init7.net:80/1/rsc_de/mp3_128");
    FavListSet(idx, NULL, "glb-stream11.streaming.init7.net:80/1/rsc_de/mp3_128");

    if ((idx = FavListSet(MAX_FAVORITES, "Radio Swiss Jazz", "zlz-stream12.streaming.init7.net:80/2/rsj/mp3_128")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "zlz-stream11.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "zlz-stream13.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream12.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream11.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "zlz-stream10.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream10.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream13.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "zlz-stream10.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "zlz-stream13.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "zlz-stream11.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream11.streaming.init7.net:80/2/rsj/mp3_128");
    FavListSet(idx, NULL, "zlz-stream12.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream10.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream12.streaming.init7.net:80/1/rsj/mp3_128");
    FavListSet(idx, NULL, "glb-stream13.streaming.init7.net:80/1/rsj/mp3_128");

    if ((idx = FavListSet(MAX_FAVORITES, ".977 The Hitz Channel", "scfire-ntc-aa04.stream.aol.com:80/stream/1074")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "scfire-mtc-aa06.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-dtc-aa03.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-mtc-aa05.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-mtc-aa04.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa03.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-mtc-aa03.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-dtc-aa02.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa02.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa07.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-mtc-aa02.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-dtc-aa05.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-dtc-aa01.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-dtc-aa04.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa08.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa05.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa01.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-mtc-aa01.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "scfire-ntc-aa06.stream.aol.com:80/stream/1074");
    FavListSet(idx, NULL, "205.188.215.230:8002");

    if ((idx = FavListSet(MAX_FAVORITES, "181.FM Power", "scfire-ntc-aa05.stream.aol.com:80/stream/1023")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "scfire-dtc-aa01.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-dtc-aa05.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-mtc-aa03.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-dtc-aa02.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-mtc-aa02.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-ntc-aa01.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-ntc-aa07.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-ntc-aa03.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-mtc-aa06.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-ntc-aa02.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-mtc-aa01.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-dtc-aa03.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-mtc-aa04.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-dtc-aa04.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-ntc-aa04.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "scfire-ntc-aa06.stream.aol.com:80/stream/1023");
    FavListSet(idx, NULL, "205.188.215.228:8002");

    if ((idx = FavListSet(MAX_FAVORITES, "181.FM The Eagle", "108.61.73.119:10030")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "108.61.73.118:10002");
    FavListSet(idx, NULL, "108.61.73.119:10000");
    FavListSet(idx, NULL, "108.61.73.118:10000");
    FavListSet(idx, NULL, "108.61.73.119:10002");
    FavListSet(idx, NULL, "108.61.73.118:10030");
    FavListSet(idx, NULL, "184.107.168.138:9972");
    FavListSet(idx, NULL, "108.61.73.119:8030");
    FavListSet(idx, NULL, "108.61.73.118:8030");

    if ((idx = FavListSet(MAX_FAVORITES, "1 Live", "gffstream.ic.llnwd.net/stream/gffstream_stream_wdr_einslive_a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_stream_wdr_einslive_b");

    if ((idx = FavListSet(MAX_FAVORITES, "1 Live diggi", "gffstream.ic.llnwd.net/stream/gffstream_einslive_a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_einslive_b");

    if ((idx = FavListSet(MAX_FAVORITES, "WDR 2", "gffstream.ic.llnwd.net/stream/gffstream_w17b")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w17a");

    if ((idx = FavListSet(MAX_FAVORITES, "WDR 3", "gffstream.ic.llnwd.net/stream/gffstream_w21a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w21b");

    if ((idx = FavListSet(MAX_FAVORITES, "WDR 4", "gffstream.ic.llnwd.net/stream/gffstream_w18b")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w18a");

    if ((idx = FavListSet(MAX_FAVORITES, "WDR 5", "gffstream.ic.llnwd.net/stream/gffstream_w19a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w19b");

    if ((idx = FavListSet(MAX_FAVORITES, "Bayern 1", "gffstream.ic.llnwd.net/stream/gffstream_w10a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w10b");

    if ((idx = FavListSet(MAX_FAVORITES, "Bayern 2", "gffstream.ic.llnwd.net/stream/gffstream_w11a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w11b");

    if ((idx = FavListSet(MAX_FAVORITES, "Bayern 3", "gffstream.ic.llnwd.net/stream/gffstream_w12a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w12b");

    if ((idx = FavListSet(MAX_FAVORITES, "BR-Klassik", "gffstream.ic.llnwd.net/stream/gffstream_w13a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w13b");

    if ((idx = FavListSet(MAX_FAVORITES, "B5 aktuell", "gffstream.ic.llnwd.net/stream/gffstream_w14a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w14b");

    if ((idx = FavListSet(MAX_FAVORITES, "B5 plus", "gffstream.ic.llnwd.net/stream/gffstream_w15a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w15b");

    if ((idx = FavListSet(MAX_FAVORITES, "Bayern plus", "gffstream.ic.llnwd.net/stream/gffstream_w16a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w16b");

    if ((idx = FavListSet(MAX_FAVORITES, "on3-radio", "gffstream.ic.llnwd.net/stream/gffstream_w9a")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "gffstream.ic.llnwd.net/stream/gffstream_w9b");

#else
    if ((idx = FavListSet(MAX_FAVORITES, "FREQUENCE3", "88.191.135.182:8010")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "193.251.154.242:8010");
    FavListSet(idx, NULL, "193.222.128.174:8010");
    FavListSet(idx, NULL, "193.251.154.243:8010");
    /* Start with this reliable station. */
    FavListCopy(idx, LAST_FAVORITE);

    if ((idx = FavListSet(MAX_FAVORITES, "Country", "173.215.30.32:9000")) == -1) {
        return;
    }

    if ((idx = FavListSet(MAX_FAVORITES, "Reggae", "84.127.17.40:8006")) == -1) {
        return;
    }

    if ((idx = FavListSet(MAX_FAVORITES, "Opera", "72.13.81.5:5046")) == -1) {
        return;
    }
    FavListSet(idx, NULL, "72.13.83.72:5046");

#endif
}

/*@}*/
